@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '0.74';
+$VERSION = '0.75';
 use base 'Exporter';
 
 our @EXPORT = qw/%builtin_types/;
@@ -395,6 +395,7 @@ sub _valid_ncname($)
    $name =~ m/^[a-zA-Z_](?:[\w.-]*)$/;
 }
 
+# better checks needed
 $builtin_types{ID} =
 $builtin_types{IDREF} =
 $builtin_types{NCName} =
@@ -406,7 +407,8 @@ $builtin_types{ENTITY} =
 
 $builtin_types{IDREFS} =
 $builtin_types{ENTITIES} =
- { parse   => \&_preserve
+ { parse   => sub { [ split ' ', shift ] }
+ , format  => sub { my $v = shift; ref $v eq 'ARRAY' ? join(' ',@$v) : $v }
  , check   => sub { $_[0] !~ m/\:/ }
  , example => 'labels'
  };
@@ -417,15 +419,17 @@ $builtin_types{Name} =
  , example => 'name'
  };
 
+
+# check required!  \c
 $builtin_types{token} =
 $builtin_types{NMTOKEN} =
  { parse   => \&_collapse
  , example => 'token'
  };
 
-
 $builtin_types{NMTOKENS} =
- { parse   => \&_preserve
+ { parse   => sub { [ split ' ', shift ] }
+ , format  => sub { my $v = shift; ref $v eq 'ARRAY' ? join(' ',@$v) : $v }
  , example => 'tokens'
  };
 
